"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createQueue = exports.execRedisCommand = exports.getRedisClient = exports.getRedisInfo = exports.ping = exports.getConnectionQueues = void 0;
const ioredis_1 = require("ioredis");
const utils_1 = require("./utils");
const bullmq_1 = require("bullmq");
const Bull = require("bull");
const responders_1 = require("./responders");
const chalk = require("chalk");
const queueNameRegExp = new RegExp("(.*):(.*):id");
const maxCount = 50000;
const maxTime = 30000;
let redisClients = {};
const getQueueKeys = (client) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, e_1, _b, _c;
    let nodes = "nodes" in client ? client.nodes('master') : [client];
    let keys = [];
    const startTime = Date.now();
    try {
        for (var _d = true, nodes_1 = __asyncValues(nodes), nodes_1_1; nodes_1_1 = yield nodes_1.next(), _a = nodes_1_1.done, !_a; _d = true) {
            _c = nodes_1_1.value;
            _d = false;
            const node = _c;
            let cursor = "0";
            do {
                const [nextCursor, scannedKeys] = yield node.scan(cursor, "MATCH", "*:*:id", "COUNT", maxCount);
                cursor = nextCursor;
                keys.push(...scannedKeys);
            } while (Date.now() - startTime < maxTime && cursor !== "0");
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (!_d && !_a && (_b = nodes_1.return)) yield _b.call(nodes_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return keys;
});
function getConnectionQueues(redisOpts, clusterNodes) {
    return __awaiter(this, void 0, void 0, function* () {
        const queues = yield execRedisCommand(redisOpts, (client) => __awaiter(this, void 0, void 0, function* () {
            const keys = yield getQueueKeys(client);
            const queues = yield Promise.all(keys
                .map(function (key) {
                var match = queueNameRegExp.exec(key);
                if (match) {
                    return {
                        prefix: match[1],
                        name: match[2],
                        type: "bull",
                    };
                }
            })
                .filter((queue) => queue !== undefined)
                .map(function (queue) {
                return __awaiter(this, void 0, void 0, function* () {
                    const type = yield (0, utils_1.getQueueType)(queue.name, queue.prefix, client);
                    queue.type = type;
                    return queue;
                });
            }));
            return queues;
        }), clusterNodes);
        return queues;
    });
}
exports.getConnectionQueues = getConnectionQueues;
function ping(redisOpts, clusterNodes) {
    return __awaiter(this, void 0, void 0, function* () {
        return execRedisCommand(redisOpts, (client) => client.ping(), clusterNodes);
    });
}
exports.ping = ping;
function getRedisInfo(redisOpts, clusterNodes) {
    return __awaiter(this, void 0, void 0, function* () {
        const info = yield execRedisCommand(redisOpts, (client) => client.info(), clusterNodes);
        return info;
    });
}
exports.getRedisInfo = getRedisInfo;
function getRedisClient(redisOpts, type, clusterNodes) {
    var _a;
    if (!redisClients[type]) {
        if (clusterNodes && clusterNodes.length) {
            const { username, password } = (0, utils_1.redisOptsFromUrl)(clusterNodes[0]);
            redisClients[type] = new ioredis_1.Redis.Cluster(clusterNodes, Object.assign(Object.assign({}, redisOpts), { redisOptions: {
                    username,
                    password,
                    tls: process.env.REDIS_CLUSTER_TLS ? {
                        cert: Buffer.from((_a = process.env.REDIS_CLUSTER_TLS) !== null && _a !== void 0 ? _a : '', 'base64').toString('ascii')
                    } : undefined,
                } }));
        }
        else {
            redisClients[type] = new ioredis_1.Redis(redisOpts);
        }
        redisClients[type].on("error", (err) => {
            console.log(`${chalk.yellow("Redis:")} ${chalk.red("redis connection error")} ${err.message}`);
        });
        redisClients[type].on("connect", () => {
            console.log(`${chalk.yellow("Redis:")} ${chalk.green("connected to redis server")}`);
        });
        redisClients[type].on("end", () => {
            console.log(`${chalk.yellow("Redis:")} ${chalk.blueBright("disconnected from redis server")}`);
        });
    }
    return redisClients[type];
}
exports.getRedisClient = getRedisClient;
function execRedisCommand(redisOpts, cb, clusterNodes) {
    return __awaiter(this, void 0, void 0, function* () {
        const redisClient = getRedisClient(redisOpts, "bull", clusterNodes);
        const result = yield cb(redisClient);
        return result;
    });
}
exports.execRedisCommand = execRedisCommand;
function createQueue(foundQueue, redisOpts, opts = {}) {
    const { nodes, integrations } = opts;
    const createClient = function (type) {
        switch (type) {
            case "client":
                return getRedisClient(redisOpts, "bull", nodes);
            default:
                throw new Error(`Unexpected connection type: ${type}`);
        }
    };
    if (integrations && integrations[foundQueue.type]) {
        const integration = integrations[foundQueue.type];
        return {
            queue: integration.createQueue(foundQueue, redisOpts, nodes),
            responders: integration.responders,
        };
    }
    switch (foundQueue.type) {
        case "bullmq":
            return {
                queue: new bullmq_1.Queue(foundQueue.name, {
                    connection: getRedisClient(redisOpts, "bullmq", nodes),
                    prefix: foundQueue.prefix,
                }),
                responders: responders_1.BullMQResponders,
            };
        case "bull":
            return {
                queue: new Bull(foundQueue.name, {
                    createClient,
                    prefix: foundQueue.prefix,
                }),
                responders: responders_1.BullResponders,
            };
        default:
            console.error(chalk.red(`ERROR:`) +
                `Unexpected queue type: ${foundQueue.type} for queue ${foundQueue.name}`);
    }
}
exports.createQueue = createQueue;
