"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateQueuesCache = exports.queueKey = exports.getCache = void 0;
const lodash_1 = require("lodash");
const queue_factory_1 = require("./queue-factory");
let queuesCache = null;
const getCache = () => {
    return queuesCache;
};
exports.getCache = getCache;
function queueKey(queue) {
    return `${queue.prefix}:${queue.name}`;
}
exports.queueKey = queueKey;
function updateQueuesCache(redisOpts, opts = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        const { nodes, integrations } = opts;
        const newQueues = yield (0, queue_factory_1.getConnectionQueues)(redisOpts, nodes);
        queuesCache = queuesCache || {};
        const oldQueues = Object.keys(queuesCache);
        const newQueuesObject = (0, lodash_1.keyBy)(newQueues, (queue) => queueKey(queue));
        const toAdd = [];
        const toRemove = [];
        for (let i = 0; i < newQueues.length; i++) {
            const newQueue = newQueues[i];
            const oldQueue = queuesCache[queueKey(newQueue)];
            if (!oldQueue) {
                toAdd.push(newQueue);
            }
        }
        for (let i = 0; i < oldQueues.length; i++) {
            const oldQueue = oldQueues[i];
            const newQueue = newQueuesObject[oldQueue];
            if (!newQueue) {
                toRemove.push(queuesCache[oldQueue]);
            }
        }
        yield Promise.all(toRemove.map(function ({ queue }) {
            const closing = queue.close();
            const name = queue["name"];
            delete queuesCache[name];
            return closing;
        }));
        toAdd.forEach(function (foundQueue) {
            const key = queueKey(foundQueue);
            const queue = (0, queue_factory_1.createQueue)(foundQueue, redisOpts, { nodes, integrations });
            if (queue) {
                queuesCache[key] = queue;
            }
        });
        return newQueues.filter((queue) => !!queuesCache[queueKey(queue)]);
    });
}
exports.updateQueuesCache = updateQueuesCache;
