"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BullMQResponders = void 0;
const respond_1 = require("./respond");
function paginate(ws, queue, messageId, start, end, method, opts) {
    start = start || 0;
    end = end || -1;
    return queue[method](start, end, opts).then(function (jobs) {
        (0, respond_1.respond)(ws, messageId, jobs);
    });
}
function respondJobCommand(ws, queue, msg) {
    return __awaiter(this, void 0, void 0, function* () {
        const data = msg.data;
        const job = yield queue.getJob(data.jobId);
        switch (data.cmd) {
            case "retry":
                yield job.retry();
                break;
            case "promote":
                yield job.promote();
                break;
            case "remove":
                yield job.remove();
                break;
            case "discard":
                yield job.discard();
                break;
            case "moveToFailed":
                yield job.moveToFailed(new Error("Failed manually"), "0");
                break;
            case "update":
                yield job.updateData(data.data);
            default:
                console.error(`Missing command ${data.cmd}. Too old version of taskforce-connector?`);
        }
        (0, respond_1.respond)(ws, msg.id);
    });
}
function respondQueueCommand(ws, queue, msg) {
    return __awaiter(this, void 0, void 0, function* () {
        const data = msg.data;
        switch (data.cmd) {
            case "getJob":
                const job = yield queue.getJob(data.jobId);
                (0, respond_1.respond)(ws, msg.id, job);
                break;
            case "getJobCounts":
                const jobCounts = yield queue.getJobCounts();
                (0, respond_1.respond)(ws, msg.id, jobCounts);
                break;
            case "getMetrics":
                const metrics = yield queue.getMetrics(data.type, data.start, data.end);
                (0, respond_1.respond)(ws, msg.id, metrics);
                break;
            case "getWaiting":
            case "getActive":
            case "getDelayed":
            case "getCompleted":
            case "getFailed":
            case "getRepeatableJobs":
            case "getWorkers":
                paginate(ws, queue, msg.id, data.start, data.end, data.cmd, data.opts);
                break;
            case "getJobLogs":
                const logs = yield queue.getJobLogs(data.jobId, data.start, data.end);
                (0, respond_1.respond)(ws, msg.id, logs);
            case "getWaitingCount":
            case "getActiveCount":
            case "getDelayedCount":
            case "getCompletedCount":
            case "getFailedCount":
            case "getRepeatableCount":
            case "getWorkersCount":
                const count = yield queue[data.cmd]();
                (0, respond_1.respond)(ws, msg.id, count);
                break;
            case "removeRepeatableByKey":
                yield queue.removeRepeatableByKey(data.key);
                (0, respond_1.respond)(ws, msg.id);
                break;
            case "add":
                const [name, jobData, opts] = data.args;
                yield queue.add(name, jobData, opts);
                (0, respond_1.respond)(ws, msg.id);
                break;
            case "empty":
                yield queue.drain();
                (0, respond_1.respond)(ws, msg.id);
                break;
            case "pause":
                yield queue.pause();
                (0, respond_1.respond)(ws, msg.id);
                break;
            case "resume":
                yield queue.resume();
                (0, respond_1.respond)(ws, msg.id);
                break;
            case "isPaused":
                const isPaused = yield queue.isPaused();
                (0, respond_1.respond)(ws, msg.id, isPaused);
                break;
            case "obliterate":
                yield queue.obliterate();
                (0, respond_1.respond)(ws, msg.id);
                break;
            case "clean":
                yield queue.clean(data.grace, data.limit, data.status);
                (0, respond_1.respond)(ws, msg.id);
                break;
            case "retryJobs":
                yield queue.retryJobs({
                    status: data.status,
                    count: data.count,
                });
                (0, respond_1.respond)(ws, msg.id);
                break;
            default:
                console.error(`Missing command ${data.cmd}. Too old version of taskforce-connector?`);
                (0, respond_1.respond)(ws, msg.id, null);
        }
    });
}
exports.BullMQResponders = {
    respondJobCommand,
    respondQueueCommand,
};
