"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.redisOptsFromUrl = exports.getQueueType = void 0;
const url = require("url");
const getQueueType = (queueName, prefix, client) => __awaiter(void 0, void 0, void 0, function* () {
    const metaKey = `${prefix}:${queueName}:meta`;
    const hasMeta = yield client.exists(metaKey);
    if (hasMeta) {
        const version = yield client.hget(metaKey, "version");
        if (version && version.includes("bullmq-pro")) {
            return "bullmq-pro";
        }
        const maxLenEvents = yield client.hget(metaKey, "opts.maxLenEvents");
        if (maxLenEvents) {
            return "bullmq";
        }
    }
    return "bull";
});
exports.getQueueType = getQueueType;
function redisOptsFromUrl(urlString) {
    const redisOpts = {};
    try {
        const redisUrl = url.parse(urlString);
        redisOpts.port = parseInt(redisUrl.port) || 6379;
        redisOpts.host = redisUrl.hostname;
        redisOpts.db = redisUrl.pathname
            ? parseInt(redisUrl.pathname.split("/")[1])
            : 0;
        if (redisUrl.auth) {
            const username = redisUrl.auth.split(":")[0];
            redisOpts.username = username ? username : undefined;
            redisOpts.password = redisUrl.auth.split(":")[1];
        }
    }
    catch (e) {
        throw new Error(e.message);
    }
    return redisOpts;
}
exports.redisOptsFromUrl = redisOptsFromUrl;
